from lambdatool.lambda_creator import LambdaCreator #noqa
from lambdatool.lambda_deployer import LambdaDeployer #noqa
from datetime import datetime

__title__ = 'lambdatool'
__version__ = '0.9.3'
__author__ = 'Mr. Chuck Muckamuck'
__author_email__ = 'chuck.muckamuck@gmail.com'
__license__ = '''MIT License

Copyright (c) 2017 Chuck Muckamuck

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
'''
__copyright__ = '(c) {}'.format(datetime.now().year)
__url__ = 'https://github.com/muckamuck/stackility'
__description__ = '''Sed sit amet congue augue. Phasellus aliquam leo non orci
egestas, in luctus enim lacinia. Praesent vulputate enim at placerat pharetra.
Aenean ullamcorper accumsan eros, ac porttitor nulla eleifend sed. In nunc
quam, fringilla sit amet sollicitudin eget, varius vel est. Phasellus mollis
libero eget sapien rhoncus, semper ultrices leo imperdiet. Suspendisse interdum
finibus turpis, id consequat quam pellentesque volutpat. Aenean diam ex,
tincidunt et lorem non, bibendum consequat nunc.
'''
